import { promisify } from "util";
import fs from 'fs';
import path from 'path';
import parseFrontMatter, { GrayMatterFile } from 'gray-matter';

const readdir = promisify(fs.readdir);
const readfile = promisify(fs.readFile);

export type Content<Data> = GrayMatterFile<string> & { filename: string; data: Data };

export async function getContent<Data = object>(dir = 'content'): Promise<Array<Content<Data>>> {
  const dirContents = await readdir(dir);
  const contentFiles = dirContents.filter(filename => filename.substring(filename.length - '.md'.length) === '.md');
  const rawContent = await Promise.all(contentFiles.map((postFile) => readfile(path.resolve(dir, postFile), 'utf-8')));
  const content = rawContent
    .map((content) => parseFrontMatter(content))
    .map((contentData, index) => ({ ...contentData, filename: contentFiles[index] })) as Array<Content<Data>>;

  return content;
}
